@@ -1,5 +1,16 @@
 import pandas as pd
 import numpy as np 
+import getpass
+
+# set the sf account name
+sf_account = 'ra45066.eu-west-1'
+from toolkit_w.snowflake.snowflakeq import Snowflakeq
+SQ=Snowflakeq()
+
+pwd = getpass.getpass("passsword:")
+
+
+
 
 
 class Crp:
@@ -134,7 +145,7 @@ class Crp:
 
 
 
-    def getLastOrderValue(self,matrix,train_df):
+    def getLastOrderparams(self,matrix,train_df):
         """
         Params:
             matrix      ->
@@ -144,8 +155,11 @@ class Crp:
             output = matrix with last order value
         """
         
-        target = train_df[train_df.customerid.map(matrix['order_date']) == train_df.order_date][['customerid','totalprice']]
-        matrix = matrix.reset_index()
+        target = train_df[train_df.customerid.map(matrix['order_date']) == train_df.order_date][['customerid',
+        'totalprice','totaldiscounts','totalweight']]
+        matrix = matrix.reset_index().rename(columns = {'totalprice':'LastOrderValue',
+                                'totaldiscounts':'LastOrderDiscount',
+                                'totalweight':'LastOrderWeight'})
         output = pd.merge(target,matrix,on='customerid')
         # output.drop_duplicates(inplace = True)
         
@@ -157,6 +171,8 @@ class Crp:
     def getTotalSpentLMonth(self,train_df,dateFlag_sep,df_matrix):
         """
         Params:
+
+
         -----------------
         Output:
         
@@ -169,18 +185,16 @@ class Crp:
         
         last_month_df = train_df[(train_df.order_date >= total_spent_last_month_sep - pd.DateOffset(months = 1)) & (train_df.order_date <= total_spent_last_month_sep)]
         
-        last_month_spent = last_month_df.groupby('customerid').agg({'totalprice':'sum'}).reset_index()
+        last_month_spent = last_month_df.groupby('customerid').agg({'totalprice':
+                                                        'sum'}).reset_index().rename(columns = {'totalprice':'LastMonthTotalSPent'})
         
-
         df_matrix = df_matrix.reset_index()
 
         df_matrix = pd.merge(df_matrix,last_month_spent,on='customerid',how = 'left')
         
-        
-        
         df_matrix = df_matrix.fillna(0)
         
-        df_matrix = df_matrix.rename(columns = {'totalprice':'Last_Month_Total_Spent'})
+        # df_matrix = df_matrix.rename(columns = {'totalprice':'Last_Month_Total_Spent'})
         
         
         
@@ -191,37 +205,41 @@ class Crp:
 
 # ----------------------------------------------------------------------------------
 
-    def getFeaturesMatrix(self,train_df,org_df,timeFlag,items_dataset):
+    def getFeaturesMatrix(self,train_df,org_df,timeFlag,items_dataset,customer_name,train_window):
         """
         Params:
             train_df -> train timeframe dataframe
             org_df   -> original dataframe with the all the needed data
-            timeFlag -> date separator ( between the traget period and the train period)
+            timeFlag -> date separator (between the traget period and the train period)
         ----------------
         Output: Features Matrix of the chosen timeframe
-        
         """
         
         # Get customer ids
         customers = train_df['customerid'].unique()
         # Get the date which acts as reference point for future calculations
         dateFlag_sep = np.max(org_df['order_date']) - pd.DateOffset(months=timeFlag)
+        # The date which is the min date for the feature matrix 
+        date_flag_min = dateFlag_sep - pd.DateOffset(months=train_window)
         # Create a grouping aggregator
         grp = train_df.groupby('customerid')
 
-        grp_dict = {'order_date':'max','id':'count','totalprice':'sum','buyeracceptsmarketing':'max','billingaddresscountrycode':'unique'}
+        grp_dict = {'order_date':'max','id':'count','totalprice':'sum','buyeracceptsmarketing':'max',
+                    'billingaddresscountrycode':'unique'}
 
         df_matrix = grp.agg(grp_dict).rename(columns={'id':'num_orders','totalprice':'TotalSpent'})
-
+        # Recency Feature
         df_matrix = self.getRecency(df_matrix,dateFlag_sep)
-
-        df_matrix = self.getLastOrderValue(df_matrix,train_df)
-
+        # Different Last orders parameters feature ( Discount value, weight ) 
+        df_matrix = self.getLastOrderparams(df_matrix,train_df)
+        # Total expenduture of the last month before date separator
         df_matrix = self.getTotalSpentLMonth(train_df,dateFlag_sep,df_matrix)
 
 
+        orders_param = SQ.get_data_from_sf('V_DIFFDAYSBETWEENPURCHASESAGG',customer_name,pwd,dateFlag_sep,date_flag_min)
+
+        output = pd.merge(df_matrix,orders_param,on='customerid')
         
-        output = df_matrix
         
         return output
 
@@ -250,7 +268,7 @@ class Crp:
 
 
 
-    def prepareDF(self,original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset):
+    def prepareDF(self,original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window):
         """
         Params:
             original_dataset ->
@@ -258,7 +276,7 @@ class Crp:
             train_window     ->
         -------------------------------__version__
         Output:
-        
+        :")
         """
         
         # First, perform initial cleaning and save a log file of action
@@ -274,7 +292,7 @@ class Crp:
         target_value = self.getTargetValues(train_data,target_frame)
         print('target value done')
         # Calculate features
-        df = self.getFeaturesMatrix(train_data,original_dataset,lag,items_dataset)
+        df = self.getFeaturesMatrix(train_data,original_dataset,lag,items_dataset,customer_name,train_window)
         
         # Merge feature sets
         # data = self.getMergedDF(df,'customerid')
@@ -290,12 +308,12 @@ class Crp:
         return output
 # -----------------------------------------------------------------------------------------------
 
-    def buildDF(self,original_dataset,lag,train_window,items_dataset):
+    def buildDF(self,original_dataset,lag,train_window,items_dataset,customer_name):
         """
         Params:
             original_dataset - > the original raw data 
             lag              - > size ( in months) of the targetValue window
-            train_window     - > size (__version__------------
+            train_window     - > size of the feature matrix
         Output: Constructed dataframe 
         
         """
@@ -312,14 +330,14 @@ class Crp:
         date_flag_min = dateFlag_sep - pd.DateOffset(months=train_window)
         print(dateFlag_sep,date_flag_min)
         # Construct dataframe according to that timeframe
-        output.append(self.prepareDF(original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset))
+        output.append(self.prepareDF(original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window))
         print(dateFlag_sep,date_flag_min)
         # Repeat the procedure until the bottom limit of the trainFrame reaches the minimum date possible according to the original dataset.
         while date_flag_min >= min_date:
         
             dateFlag_sep = dateFlag_sep - pd.DateOffset(months=lag)
             date_flag_min = dateFlag_sep - pd.DateOffset(months=train_window)
-            output.append(self.prepareDF(original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset))
+            output.append(self.prepareDF(original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window))
             print(dateFlag_sep,date_flag_min)
             
             
